package api

import (
	"errors"
	"fmt"
	"sku3dweb/bus"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"sku3dweb/log"
	"strings"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"infish.cn/comm"
)

func RegShareApis(router *GinRouter) {

	router.POSTJWT("/share/pack/create", createShare)

	router.GET("/share/pack/detail/:id", packShareDetail)
}

func packShareDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	if len(id) < 1 {
		return nil, fmt.Errorf("id不能为空")
	}
	repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), "share", id, bson.M{"$inc": bson.M{"pv": 1}, "$set": bson.M{"lastTime": time.Now()}})

	share := &model.Share{}
	ok, _ := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		Query:       repo.Map{"_id": id},
		CollectName: "share",
	}, share)
	if !ok {
		return nil, fmt.Errorf("数据已失效")
	}

	// 如果是面料类型
	if strings.Contains(share.Coll, "mat") {
		matInfo, err := bus.BusTreeAssetMatgroupDetail(share.DbName, share.Coll, share.AssetId.Hex())
		if err != nil {
			fmt.Println(err)
			log.Error(err)
			return nil, errors.New("获取资源信息错误！")
		}
		if share.UserId.IsZero() {
			matInfo.QiYeInfo = &comm.QiYeInfo{}
			return matInfo, nil

		}
		matInfo.QiYeInfo, err = getCompanyInfo(share.UserId, apictx)
		if err != nil {
			log.Error(err)
			fmt.Println(err)
		}
		return matInfo, nil
	}

	// return bus.BusTreeAssetPackDetail(share.DbName, share.Coll, share.AssetId.Hex())
	// !11.06添加企业信息
	assetInfo, err := bus.BusTreeAssetPackDetail(share.DbName, share.Coll, share.AssetId.Hex())
	if err != nil {
		fmt.Println(err)
		log.Error(err)
		return nil, errors.New("获取资源信息错误！")
	}
	if share.UserId.IsZero() {
		assetInfo.QiYeInfo = &comm.QiYeInfo{}
		return assetInfo, nil

	}
	assetInfo.QiYeInfo, err = getCompanyInfo(share.UserId, apictx)
	if err != nil {
		log.Error(err)
		fmt.Println(err)
	}
	return assetInfo, nil
}

func getCompanyInfo(ownerId primitive.ObjectID, apictx *ApiSession) (*comm.QiYeInfo, error) {
	user := model.User{}
	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		Db:          "usercenter",
		CollectName: "users",
		Query:       repo.Map{"_id": ownerId},
		Project:     []string{"qiye", "parent", "name"},
	}, &user)
	if !ok || err != nil {
		return &comm.QiYeInfo{}, errors.New("查询企业信息失败！")
	}
	// 没有母账号，就是自己的
	if len(user.Parent) < 1 {
		if user.Qiye == nil {
			return &comm.QiYeInfo{Id: user.Id, Name: user.Name}, nil
		}
		return &comm.QiYeInfo{Id: user.Id, Name: user.Qiye.Name}, nil

	}
	// 有母账号，查询母账号信息
	ok, err = repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		Db:          "usercenter",
		CollectName: "users",
		Query:       repo.Map{"_id": user.Parent},
		Project:     []string{"qiye", "name"},
	}, &user)
	if !ok || err != nil {
		return &comm.QiYeInfo{}, errors.New("查询企业信息失败！")
	}
	parent, _ := primitive.ObjectIDFromHex(user.Parent)
	if user.Qiye == nil {
		return &comm.QiYeInfo{Id: user.Id, Name: user.Name}, nil
	}
	return &comm.QiYeInfo{Id: parent, ChildId: user.Id, Name: user.Qiye.Name}, nil

}

func createShare(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	body := &struct {
		Id        string
		Scope     string
		AssetType string
	}{}
	err := c.ShouldBindJSON(body)
	if err != nil {
		return nil, err
	}

	if len(body.Id) < 1 || len(body.Scope) < 1 {
		return nil, fmt.Errorf("参数不对")
	}

	isCompany := model.USERTYPE_COMPANY == apictx.User.UserType
	isSku3d := apictx.User.Parent == model.SKU3D_USERID

	//from collection 2
	Coll := getAssetTypeCollectin(isCompany, isSku3d, body.Scope, body.AssetType)

	uid, _ := primitive.ObjectIDFromHex(apictx.User.ID)
	assetId, _ := primitive.ObjectIDFromHex(body.Id)

	share := &model.Share{}

	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: "share",
		// !11.15 缺少条件导致已存在的 platform_shoe_user 数据 代替该生成的 platform_shoe_team数据，"coll":Coll
		Query: repo.Map{"userId": uid, "assetId": assetId, "coll": Coll},
	}, share)
	if err != nil {
		return nil, err
	}
	if ok {
		return share.Id.Hex(), nil
	}

	share.AssetId = assetId
	share.UserId = uid
	share.DbName = Sku3dDbName
	share.Coll = Coll
	share.AssetType = body.AssetType
	share.Scope = body.Scope
	share.PV = 1
	share.CreateTime = time.Now()

	return repo.RepoAddDoc(apictx.CreateRepoCtx(), "share", share)
}

func ServiceShareDesign(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	body := struct {
		Id  string
		Pwd string
	}{}

	body.Id = c.Query("id")
	body.Pwd = c.Query("pwd")

	if len(body.Id) < 1 {
		return nil, NewError("Id不能为空")
	}

	did, _ := primitive.ObjectIDFromHex(body.Id)

	option := repo.DocSearchOptions{
		CollectName: repo.CollectionDesigns,
		Query:       repo.Map{"_id": did},
		Project:     []string{"name", "thumbnail", "editor", "scenes", "shoeId", "createTime", "sharepwd"},
	}
	design := &model.Design{}
	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &option, design)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, NewError("项目Id无效!")
	}

	if len(design.Sharepwd) > 0 && body.Pwd != design.Sharepwd { //不相等，需要输入密码
		return nil, NewErrorWithCode("401-nopwd", 401)
	}

	option = repo.DocSearchOptions{
		CollectName: repo.CollectionShoes,
		Query:       repo.Map{"_id": design.ShoeId},
		Project:     []string{"name", "materials", "leftComponents", "meshId"},
	}
	shoe := &model.ShoeMesh{}
	ok, err = repo.RepoSeachDoc(apictx.CreateRepoCtx(), &option, shoe)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, NewError("shoeId无效!")
	}

	//查询osgjs
	option = repo.DocSearchOptions{
		CollectName: repo.CollectionMesh,
		Query:       repo.Map{"_id": shoe.MeshId},
		Project:     []string{"osgjs"},
	}
	mesh := &model.Mesh{}
	ok, err = repo.RepoSeachDoc(apictx.CreateRepoCtx(), &option, mesh)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, NewError("meshId无效!")
	}

	//获取所有面料
	fabric := []*model.Fabric{}
	fabricMap := map[string]bool{}

	//获取所有色卡
	material := []*model.Material{}
	materialMap := map[string]bool{}

	//env3d
	env3d := []map[string]interface{}{}
	env3dMap := map[string]bool{}

	//xxx
	for _, item := range design.Scenes {

		if !env3dMap[item.Env.Id.Hex()] {
			option = repo.DocSearchOptions{
				CollectName: repo.CollectionEvn3d,
				Query:       repo.Map{"_id": item.Env.Id},
				Project:     []string{"name", "thumbnail", "config"},
			}
			ok, env := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &option)
			if ok {
				env3d = append(env3d, env)
				env3dMap[item.Env.Id.Hex()] = true
			}
		}

		//色卡和面料
		for _, comp := range item.Components {
			if comp.FabricId != nil && !comp.FabricId.IsZero() {
				fid := comp.FabricId.Hex()
				if !fabricMap[fid] {
					fb := repo.FabricGetById(apictx.CreateRepoCtx(), comp.FabricScope, fid, []string{"name"})
					if fb != nil {
						fabricMap[fid] = true
						fabric = append(fabric, fb)
					}
				}
			}

			if comp.CardId != nil && !comp.CardId.IsZero() {
				fid := comp.CardId.Hex()
				if !materialMap[fid] {
					mat := repo.MaterialGetById(apictx.CreateRepoCtx(), fid, []string{"name"})
					if mat != nil {
						materialMap[fid] = true
						material = append(material, mat)
					}
				}
			}
		}
	}

	return map[string]interface{}{
		"env3d":    env3d,
		"fabric":   fabric,
		"material": material,
		"design":   design,
		"shoe":     shoe,
		"mesh":     mesh,
	}, nil
}
