package api

import (
	"fmt"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"sku3dweb/log"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

const DefaultUserAvatar = "https://os.alipayobjects.com/rmsportal/QBnOOoLaAfKPirc.png"

func CreateQiyeUserRouter(router *GinRouter) {

	//添加子账号
	router.POSTJWT("/user/subaccount/create", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		account := &model.SubAccount{}
		c.ShouldBindJSON(account)
		if len(account.Password) < 1 || len(account.Phone) < 1 {
			return nil, NewError("phone或者密码不能为空")
		}
		account.Password = UtilMd5(account.Password)
		account.CreateTime = time.Now()

		//判断是否有同名子的子账号
		user := &model.User{}
		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: repo.CollectionUser,
			Query:       repo.Map{"phone": account.Phone},
			Project:     []string{"name", "avatar", "phone"},
		}, user)

		if err != nil {
			return nil, err
		}

		if ok {
			if user.Parent == apictx.User.Parent { //已经是子账号了
				return nil, NewError("该子账号已经存在!")
			}
			return nil, NewError("该用户不能作为当前账号子账号!")
		}

		//没有对应的账号,可用添加
		user = &model.User{Name: account.Name, IsPlatform: BoolValue(false), Parent: apictx.User.Parent, Password: account.Password, Avatar: DefaultUserAvatar, Phone: account.Phone, CreateTime: time.Now()}
		user.Roles = account.RoleIds
		user.TeamId = account.TeamId

		uid, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionUser, user)
		if err != nil {
			log.Error(err)
			return nil, NewError("添加用户失败!")
		}

		return uid, nil
	})

	//查询子账号
	router.GETJWT("/user/subaccount/list", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

		ok, list := repo.RepoSeachDocsMap(apictx.CreateRepoCtx(), &repo.DocsSearchOptions{
			CollectName: repo.CollectionUser,
			Query:       repo.Map{"parent": apictx.User.Parent},
			Project:     []string{"name", "avatar", "phone", "createTime", "lastLogin", "roles", "isDeleted", "teamId"},
		})
		if !ok {
			return []string{}, nil
		}

		//查询team
		teamNames := map[string]string{}
		for _, user := range list {
			if user["teamId"] != nil {
				teamId := user["teamId"].(string)
				if len(teamNames[teamId]) > 0 {
					user["team"] = teamNames[teamId]
					continue
				}

				team := &model.Team{}
				ok, _ := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{CollectName: repo.CollectionTeam, Query: repo.Map{"_id": teamId}, Project: []string{"name"}}, team)
				if ok {
					teamNames[teamId] = team.Name
					user["team"] = teamNames[teamId]
				}
			}
		}
		return list, nil
	})

	//删除子账号
	router.POSTJWT("/user/subaccount/delete/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		id := c.Param("id")
		if len(id) < 1 {
			return nil, NewError("id不能为空！")
		}
		return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionUser, id)
	})

	//编辑子账号
	router.POSTJWT("/user/subaccount/update", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		account := &struct {
			Id       string
			Phone    string
			Password string
			Name     string
			TeamId   string
			Roles    []string
		}{}
		c.ShouldBindJSON(account)

		if len(account.Id) < 1 {
			return nil, NewError("Id不能为空")
		}

		if len(account.Password) > 0 {
			account.Password = UtilMd5(account.Password)
		}

		if len(account.Phone) > 0 {
			ok, list := repo.RepoSeachDocsMap(apictx.CreateRepoCtx(), &repo.DocsSearchOptions{
				CollectName: repo.CollectionUser,
				Query:       repo.Map{"phone": account.Phone},
				Project:     []string{"name", "avatar", "phone", "_id"},
			})
			if ok && len(list) == 1 {
				if list[0]["_id"].(primitive.ObjectID).Hex() != account.Id {
					return nil, NewError("当前手机号已存在")
				}
			}
		}

		return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionUser, account.Id, &model.User{Name: account.Name, TeamId: account.TeamId, Phone: account.Phone, Password: account.Password, Roles: account.Roles})
	})

	//编辑子账号权限
	router.POSTJWT("/user/subaccount/role", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		account := &struct {
			Id    string
			Roles []string
		}{}
		c.ShouldBindJSON(account)
		if len(account.Id) < 1 {
			return nil, NewError("Id不能为空")
		}
		if len(account.Roles) < 1 {
			return nil, NewError("权限不能为空")
		}

		return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionUser, account.Id, &model.User{Roles: account.Roles})
	})

	//编辑子账号权限
	router.GETJWT("/user/roles", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

		ok, result := repo.RepoSeachDocsMap(apictx.CreateRepoCtx(), &repo.DocsSearchOptions{CollectName: repo.CollectionRoles, Query: repo.Map{"userId": apictx.User.Parent}, Project: []string{"name", "perms", "createTime"}})
		if !ok {
			return nil, NewError("查询失败!")
		}
		return result, nil
	})

	//添加主账号角色
	router.POSTJWT("/user/role/create", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		body := &model.Role{}
		c.ShouldBindJSON(body)
		body.UserId = apictx.User.Parent
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionRoles, body)
	})
	//更新角色内容
	router.POSTJWT("/user/role/update", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		body := &model.Role{}
		c.ShouldBindJSON(body)
		if body.Id == primitive.NilObjectID {
			return nil, NewError("id不能为空")
		}
		id := body.Id
		body.Id = primitive.NilObjectID
		return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionRoles, id.Hex(), body)
	})
	//更新角色内容
	router.POSTJWT("/user/role/remove/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		id := c.Param("id")
		if len(id) < 1 {
			return nil, NewError("id为空")
		}
		return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionRoles, id)
	})

	//企业信息边疆QiyeInfo
	router.POSTJWT("/user/qiye", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		body := &model.QiyeInfo{}
		c.ShouldBindJSON(body)
		if len(body.Name) < 1 {
			return nil, NewError("企业名称不能为空")
		}
		return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionUser, apictx.User.Parent, &model.User{Qiye: body})
	})

	//企业信息QiyeInfo
	router.GETJWT("/user/qiye", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		ok, result := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &repo.DocSearchOptions{CollectName: repo.CollectionUser, Query: repo.Map{"_id": apictx.User.Parent}, Project: []string{"qiye"}})
		if !ok {
			return nil, NewError("查询失败!")
		}
		return result, nil
	})

	CreateCRUD(router, "/team", &CRUDOption{
		Collection: repo.CollectionTeam,
		NewModel2: func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
			team := &model.Team{}
			c.ShouldBindJSON(team)
			team.UserId = apictx.User.Parent
			team.CreateTime = time.Now()
			team.UserCount = 0
			return team, nil
		},
		JWT: true,
		EmtyModel: func(*gin.Context) interface{} {
			return &model.Team{}
		},
		SearchFilter: func(c *gin.Context, apictx *ApiSession, query map[string]interface{}) map[string]interface{} {
			return map[string]interface{}{"userId": apictx.User.Parent}
		},
		SearchPostProcess: func(page *repo.PageResult, c *gin.Context, apictx *ApiSession, query map[string]interface{}) (interface{}, error) {
			//查询team的人数
			for _, team := range page.List {
				teamId := team["_id"].(primitive.ObjectID).Hex()
				fmt.Println("team=>", teamId)
				teamCount, _ := repo.RepoCountDoc(apictx.CreateRepoCtx(), repo.CollectionUser, repo.Map{"teamId": teamId})
				team["userCount"] = teamCount
				leader := team["leaderId"]
				if leader != nil {
					leaderId := leader.(string)
					ok, leaderUser := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &repo.DocSearchOptions{CollectName: repo.CollectionUser, Query: repo.Map{"_id": leaderId}, Project: []string{"name"}})
					if ok {
						team["leader"] = leaderUser
					}
				}
			}
			return page, nil
		},
		SearchProject: []string{"name", "userCount", "filters", "createTime"},
	})

	//查询team的所有人员
	router.GETJWT("/team/users/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		teamId := c.Param("id")
		if len(teamId) < 1 {
			return nil, NewError("teamId 为空!")
		}

		ok, list := repo.RepoSeachDocsMap(apictx.CreateRepoCtx(), &repo.DocsSearchOptions{
			CollectName: repo.CollectionUser,
			Query:       repo.Map{"teamId": teamId},
			Project:     []string{"name", "avatar"},
		})
		if !ok {
			return nil, NewError("查询失败!")
		}

		team := &model.Team{}

		repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: repo.CollectionTeam,
			Query:       repo.Map{"_id": teamId},
			Project:     []string{"leaderId"},
		}, team)

		return map[string]interface{}{
			"list":     list,
			"leaderId": team.LeaderId,
		}, nil
	})

	//设置team leader
	router.POSTJWT("/team/loader/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		teamId := c.Param("id")
		if len(teamId) < 1 {
			return nil, NewError("teamId 为空!")
		}
		body := &struct {
			Id string
		}{}
		c.ShouldBindJSON(body)
		if len(body.Id) < 1 {
			return nil, NewError("Leader Id 不能为空!")
		}
		return repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionTeam, teamId, bson.M{"$set": bson.M{"leaderId": body.Id}})
	})

	//移除用户
	router.POSTJWT("/team/user/remove/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		teamId := c.Param("id")
		if len(teamId) < 1 {
			return nil, NewError("teamId 为空!")
		}
		body := &struct {
			Id string
		}{}
		c.ShouldBindJSON(body)
		if len(body.Id) < 1 {
			return nil, NewError("Leader Id 不能为空!")
		}

		return repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionUser, body.Id, bson.M{"$set": bson.M{"teamId": ""}})
	})

	//联系我们
	router.POST("/company/info", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		info := &model.ContentInfo{}
		err := c.ShouldBindJSON(info)
		if err != nil {
			return nil, NewError("参数错误")
		}
		info.CreateTime = time.Now()
		ret, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionContentInfo, info)
		if err != nil {
			return nil, err
		}

		SendSmsRegNotify("15208364621", info)

		SendSmsRegNotify("15281029319", info)

		return ret, nil
	})
}
